#include "game.h"

static const bool printGame = false;

Game::Game(Player* x, Player* o) : o(o), x(x)
{
}

Game::~Game()
{
}

void Game::init()
{
   for(int i = 0; i < 3; i++) {
      for(int j = 0; j < 3; j++) {
         state[i][j] = 0;
      }
   }
}

string Game::constructState(int turn)
{
   string ret = "";
   for(int i = 0; i < 3; i++) {
      for(int j = 0; j < 3; j++) {
         switch(state[i][j]) {
            case 1:  // X
               if(turn == 0) {   // switch around, as player sees always X as himself
                  ret += "x";
               } else {
                  ret += "o";
               }
               break;
            case 2:  // O
               if(turn == 0) {
                  ret += "o";
               } else {
                  ret += "x";
               }
               break;
            case 0:
            default:
               ret += "_";
               break;
         }
      }
   }
   return ret;
}

/// should be like 1,2 for "place mark at 1st row, 2nd column"
void Game::applyAction(string action, int turn)
{
   char row = action[0];
   char col = action[2];
   int i = 0, j = 0;
   switch(row) {
      case '1':
         i = 0;
         break;
      case '2':
         i = 1;
         break;
      case '3':
         i = 2;
         break;
      default:
         return;
   }
   switch(col) {
      case '1':
         j = 0;
         break;
      case '2':
         j = 1;
         break;
      case '3':
         j = 2;
         break;
      default:
         return;
   }
   if(state[i][j] != 0) {
      return;
   }
   state[i][j] = turn + 1;
}

/// true, if finished, winner is 1 for x, 2 for o, 0 for draw
bool Game::checkDone(int & winner)
{
   winner = 0;

   bool xwin = false;
   bool owin = false;
   // row
   for(int i = 0; i < 3; i++) {
      int cx = 0;
      int co = 0;
      for(int j = 0; j < 3; j++) {
         if(state[i][j] == 1)
            cx ++;
         else if(state[i][j] == 2)
            co ++;
      }
      if(cx == 3)
         xwin = true;
      if(co == 3)
         owin = true;
   }
   // col 
   for(int j = 0; j < 3; j++) {
      int cx = 0;
      int co = 0;
      for(int i = 0; i < 3; i++) {
         if(state[i][j] == 1)
            cx ++;
         else if(state[i][j] == 2)
            co ++;
      }
      if(cx == 3)
         xwin = true;
      if(co == 3)
         owin = true;
   }

   // diag
   int cx = 0;
   int co = 0;
   for(int ii = 0; ii < 3; ii++) {
      if(state[ii][ii] == 1)
         cx ++;
      else if(state[ii][ii] == 2)
         co ++;
   }
   if(cx == 3)
      xwin = true;
   if(co == 3)
      owin = true;

   cx = 0;
   co = 0;
   for(int ii = 0; ii < 3; ii++) {
      if(state[ii][2 - ii] == 1)
         cx ++;
      else if(state[ii][2 - ii] == 2)
         co ++;
   }
   if(cx == 3)
      xwin = true;
   if(co == 3)
      owin = true;

   if(xwin && !owin) {
      winner = 1;
   } else if(owin && !xwin) {
      winner = 2;
   }

   int used = 0;
   for(int i = 0; i < 3; i++) {
      for(int j = 0; j < 3; j++) {
         if(state[i][j] != 0)
            used++;
      }
   }

   return (xwin || owin) || (used == 9);
}

void Game::play()
{
   init();

   bool done = false;
   int turn = 0;
   int winner;
   while(!done) {
      if(printGame)
         print();

      Player* p = NULL;
      if(turn == 0)
         p = x;
      else if(turn == 1)
         p = o;
      string action = p->sense(constructState(turn), 0.0);
      applyAction(action, turn);

      if(checkDone(winner))
         done = true;

      turn = 1 - turn;
   }

   if(printGame)
      print();

   if(winner == 0) {
      x->terminate(constructState(0), 0.0, true);
      o->terminate(constructState(1), 0.0);
      if(printGame)
         printf("Draw\n");
   }
   else if(winner == 1) {
      x->terminate(constructState(0), 1.0, true);
      o->terminate(constructState(1), -1.0);
      if(printGame)
         printf("X wins.\n");
   }
   else if(winner == 2) {
      x->terminate(constructState(0), -1.0, true);
      o->terminate(constructState(1), 1.0);
      if(printGame)
         printf("O wins.\n");
   }
}

void Game::print()
{
   for(int i = 0; i < 3; i++) {
      for(int j = 0; j < 3; j++) {
         switch(state[i][j]) {
            case 0:
               printf("_ ");
               break;
            case 1:
               printf("X ");
               break;
            case 2:
               printf("O ");
               break;
         }
      }
      printf("\n");
   }
   printf("\n");
}

