#ifndef PLAYER_H
#define PLAYER_H

#include <string>
using namespace std;

/// Base player class - derive from this to create your own player.
class Player
{
   public:
      Player();
      ~Player();

      /// World interface for the agent - it observerse a state, and a reward and should return an action.
      /**
       * Format: state is a 9 character string with chars 'x', 'o' or '_', where '_' marks a free spot
       * the action should output the place like 1,2 for "place mark at 1st row, 2nd column".
       */
      virtual string sense(string state, double reward) = 0;

      /// When the agent has gone into a terminal state, it does not produce any action.
      virtual void terminate(string state, double reward, bool writeLearningData = false) = 0;

      /// A player might implement load/save to store learned Q-Values in a file.
      virtual void load(string qfile) {}
      virtual void save(string qfile) {}
};

#endif

